# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.add_organization_connection_response_content import AddOrganizationConnectionResponseContent
from ...types.get_organization_connection_response_content import GetOrganizationConnectionResponseContent
from ...types.list_organization_connections_offset_paginated_response_content import (
    ListOrganizationConnectionsOffsetPaginatedResponseContent,
)
from ...types.organization_connection import OrganizationConnection
from ...types.update_organization_connection_response_content import UpdateOrganizationConnectionResponseContent
from .raw_client import AsyncRawEnabledConnectionsClient, RawEnabledConnectionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EnabledConnectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEnabledConnectionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEnabledConnectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEnabledConnectionsClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[OrganizationConnection, ListOrganizationConnectionsOffsetPaginatedResponseContent]:
        """
        Retrieve details about a specific connection currently enabled for an Organization. Information returned includes details such as connection ID, name, strategy, and whether the connection automatically grants membership upon login.

        Parameters
        ----------
        id : str
            Organization identifier.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[OrganizationConnection, ListOrganizationConnectionsOffsetPaginatedResponseContent]
            Connections successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.organizations.enabled_connections.list(
            id="id",
            page=1,
            per_page=1,
            include_totals=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            id, page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    def add(
        self,
        id: str,
        *,
        connection_id: str,
        assign_membership_on_login: typing.Optional[bool] = OMIT,
        is_signup_enabled: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddOrganizationConnectionResponseContent:
        """
        Enable a specific connection for a given Organization. To enable a connection, it must already exist within your tenant; connections cannot be created through this action.

        [Connections](https://auth0.com/docs/authenticate/identity-providers) represent the relationship between Auth0 and a source of users. Available types of connections include database, enterprise, and social.

        Parameters
        ----------
        id : str
            Organization identifier.

        connection_id : str
            Single connection ID to add to the organization.

        assign_membership_on_login : typing.Optional[bool]
            When true, all users that log in with this connection will be automatically granted membership in the organization. When false, users must be granted membership in the organization before logging in with this connection.

        is_signup_enabled : typing.Optional[bool]
            Determines whether organization signup should be enabled for this organization connection. Only applicable for database connections. Default: false.

        show_as_button : typing.Optional[bool]
            Determines whether a connection should be displayed on this organization’s login prompt. Only applicable for enterprise connections. Default: true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddOrganizationConnectionResponseContent
            Organization connection successfully added.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.enabled_connections.add(
            id="id",
            connection_id="connection_id",
        )
        """
        _response = self._raw_client.add(
            id,
            connection_id=connection_id,
            assign_membership_on_login=assign_membership_on_login,
            is_signup_enabled=is_signup_enabled,
            show_as_button=show_as_button,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, id: str, connection_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOrganizationConnectionResponseContent:
        """
        Retrieve details about a specific connection currently enabled for an Organization. Information returned includes details such as connection ID, name, strategy, and whether the connection automatically grants membership upon login.

        Parameters
        ----------
        id : str
            Organization identifier.

        connection_id : str
            Connection identifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationConnectionResponseContent
            Connection successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.enabled_connections.get(
            id="id",
            connection_id="connectionId",
        )
        """
        _response = self._raw_client.get(id, connection_id, request_options=request_options)
        return _response.data

    def delete(self, id: str, connection_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Disable a specific connection for an Organization. Once disabled, Organization members can no longer use that connection to authenticate.

        **Note**: This action does not remove the connection from your tenant.

        Parameters
        ----------
        id : str
            Organization identifier.

        connection_id : str
            Connection identifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.enabled_connections.delete(
            id="id",
            connection_id="connectionId",
        )
        """
        _response = self._raw_client.delete(id, connection_id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        connection_id: str,
        *,
        assign_membership_on_login: typing.Optional[bool] = OMIT,
        is_signup_enabled: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrganizationConnectionResponseContent:
        """
        Modify the details of a specific connection currently enabled for an Organization.

        Parameters
        ----------
        id : str
            Organization identifier.

        connection_id : str
            Connection identifier.

        assign_membership_on_login : typing.Optional[bool]
            When true, all users that log in with this connection will be automatically granted membership in the organization. When false, users must be granted membership in the organization before logging in with this connection.

        is_signup_enabled : typing.Optional[bool]
            Determines whether organization signup should be enabled for this organization connection. Only applicable for database connections. Default: false.

        show_as_button : typing.Optional[bool]
            Determines whether a connection should be displayed on this organization’s login prompt. Only applicable for enterprise connections. Default: true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrganizationConnectionResponseContent
            Organization connection successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.organizations.enabled_connections.update(
            id="id",
            connection_id="connectionId",
        )
        """
        _response = self._raw_client.update(
            id,
            connection_id,
            assign_membership_on_login=assign_membership_on_login,
            is_signup_enabled=is_signup_enabled,
            show_as_button=show_as_button,
            request_options=request_options,
        )
        return _response.data


class AsyncEnabledConnectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEnabledConnectionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEnabledConnectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEnabledConnectionsClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[OrganizationConnection, ListOrganizationConnectionsOffsetPaginatedResponseContent]:
        """
        Retrieve details about a specific connection currently enabled for an Organization. Information returned includes details such as connection ID, name, strategy, and whether the connection automatically grants membership upon login.

        Parameters
        ----------
        id : str
            Organization identifier.

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[OrganizationConnection, ListOrganizationConnectionsOffsetPaginatedResponseContent]
            Connections successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.organizations.enabled_connections.list(
                id="id",
                page=1,
                per_page=1,
                include_totals=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            id, page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    async def add(
        self,
        id: str,
        *,
        connection_id: str,
        assign_membership_on_login: typing.Optional[bool] = OMIT,
        is_signup_enabled: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddOrganizationConnectionResponseContent:
        """
        Enable a specific connection for a given Organization. To enable a connection, it must already exist within your tenant; connections cannot be created through this action.

        [Connections](https://auth0.com/docs/authenticate/identity-providers) represent the relationship between Auth0 and a source of users. Available types of connections include database, enterprise, and social.

        Parameters
        ----------
        id : str
            Organization identifier.

        connection_id : str
            Single connection ID to add to the organization.

        assign_membership_on_login : typing.Optional[bool]
            When true, all users that log in with this connection will be automatically granted membership in the organization. When false, users must be granted membership in the organization before logging in with this connection.

        is_signup_enabled : typing.Optional[bool]
            Determines whether organization signup should be enabled for this organization connection. Only applicable for database connections. Default: false.

        show_as_button : typing.Optional[bool]
            Determines whether a connection should be displayed on this organization’s login prompt. Only applicable for enterprise connections. Default: true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddOrganizationConnectionResponseContent
            Organization connection successfully added.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.enabled_connections.add(
                id="id",
                connection_id="connection_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add(
            id,
            connection_id=connection_id,
            assign_membership_on_login=assign_membership_on_login,
            is_signup_enabled=is_signup_enabled,
            show_as_button=show_as_button,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, connection_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOrganizationConnectionResponseContent:
        """
        Retrieve details about a specific connection currently enabled for an Organization. Information returned includes details such as connection ID, name, strategy, and whether the connection automatically grants membership upon login.

        Parameters
        ----------
        id : str
            Organization identifier.

        connection_id : str
            Connection identifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationConnectionResponseContent
            Connection successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.enabled_connections.get(
                id="id",
                connection_id="connectionId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, connection_id, request_options=request_options)
        return _response.data

    async def delete(
        self, id: str, connection_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Disable a specific connection for an Organization. Once disabled, Organization members can no longer use that connection to authenticate.

        **Note**: This action does not remove the connection from your tenant.

        Parameters
        ----------
        id : str
            Organization identifier.

        connection_id : str
            Connection identifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.enabled_connections.delete(
                id="id",
                connection_id="connectionId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, connection_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        connection_id: str,
        *,
        assign_membership_on_login: typing.Optional[bool] = OMIT,
        is_signup_enabled: typing.Optional[bool] = OMIT,
        show_as_button: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrganizationConnectionResponseContent:
        """
        Modify the details of a specific connection currently enabled for an Organization.

        Parameters
        ----------
        id : str
            Organization identifier.

        connection_id : str
            Connection identifier.

        assign_membership_on_login : typing.Optional[bool]
            When true, all users that log in with this connection will be automatically granted membership in the organization. When false, users must be granted membership in the organization before logging in with this connection.

        is_signup_enabled : typing.Optional[bool]
            Determines whether organization signup should be enabled for this organization connection. Only applicable for database connections. Default: false.

        show_as_button : typing.Optional[bool]
            Determines whether a connection should be displayed on this organization’s login prompt. Only applicable for enterprise connections. Default: true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrganizationConnectionResponseContent
            Organization connection successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organizations.enabled_connections.update(
                id="id",
                connection_id="connectionId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            connection_id,
            assign_membership_on_login=assign_membership_on_login,
            is_signup_enabled=is_signup_enabled,
            show_as_button=show_as_button,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .token_exchange_profile_type_enum import TokenExchangeProfileTypeEnum


class TokenExchangeProfileResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique ID of the token exchange profile.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Friendly name of this profile.
    """

    subject_token_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Subject token type for this profile. When receiving a token exchange request on the Authentication API, the corresponding token exchange profile with a matching subject_token_type will be executed. This must be a URI.
    """

    action_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the Custom Token Exchange action to execute for this profile, in order to validate the subject_token. The action must use the custom-token-exchange trigger.
    """

    type: typing.Optional[TokenExchangeProfileTypeEnum] = None
    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when this profile was created.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when this profile was updated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .phone_provider_channel_enum import PhoneProviderChannelEnum
from .phone_provider_configuration import PhoneProviderConfiguration
from .phone_provider_name_enum import PhoneProviderNameEnum


class UpdateBrandingPhoneProviderResponseContent(UniversalBaseModel):
    """
    Phone provider configuration schema
    """

    id: typing.Optional[str] = None
    tenant: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the tenant
    """

    name: PhoneProviderNameEnum
    channel: typing.Optional[PhoneProviderChannelEnum] = None
    disabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the provider is enabled (false) or disabled (true).
    """

    configuration: typing.Optional[PhoneProviderConfiguration] = None
    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The provider's creation date and time in ISO 8601 format
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The date and time of the last update to the provider in ISO 8601 format
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_org_member_role_assigned_object_organization import (
    EventStreamCloudEventOrgMemberRoleAssignedObjectOrganization,
)
from .event_stream_cloud_event_org_member_role_assigned_object_role import (
    EventStreamCloudEventOrgMemberRoleAssignedObjectRole,
)
from .event_stream_cloud_event_org_member_role_assigned_object_user import (
    EventStreamCloudEventOrgMemberRoleAssignedObjectUser,
)


class EventStreamCloudEventOrgMemberRoleAssignedObject(UniversalBaseModel):
    """
    The event content.
    """

    organization: EventStreamCloudEventOrgMemberRoleAssignedObjectOrganization
    user: EventStreamCloudEventOrgMemberRoleAssignedObjectUser
    role: EventStreamCloudEventOrgMemberRoleAssignedObjectRole

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count

from auth0.management import CreateScimConfigurationRequestContent, ScimMappingItem


def test_connections_scimConfiguration_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "connections.scim_configuration.list_.0"
    client = get_client(test_id)
    client.connections.scim_configuration.list(
        from_="from",
        take=1,
    )
    verify_request_count(test_id, "GET", "/connections-scim-configurations", {"from": "from", "take": "1"}, 1)


def test_connections_scimConfiguration_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "connections.scim_configuration.get.0"
    client = get_client(test_id)
    client.connections.scim_configuration.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/connections/id/scim-configuration", None, 1)


def test_connections_scimConfiguration_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "connections.scim_configuration.create.0"
    client = get_client(test_id)
    client.connections.scim_configuration.create(
        id="id",
        request=CreateScimConfigurationRequestContent(),
    )
    verify_request_count(test_id, "POST", "/connections/id/scim-configuration", None, 1)


def test_connections_scimConfiguration_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "connections.scim_configuration.delete.0"
    client = get_client(test_id)
    client.connections.scim_configuration.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/connections/id/scim-configuration", None, 1)


def test_connections_scimConfiguration_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "connections.scim_configuration.update.0"
    client = get_client(test_id)
    client.connections.scim_configuration.update(
        id="id",
        user_id_attribute="user_id_attribute",
        mapping=[ScimMappingItem()],
    )
    verify_request_count(test_id, "PATCH", "/connections/id/scim-configuration", None, 1)


def test_connections_scimConfiguration_get_default_mapping() -> None:
    """Test getDefaultMapping endpoint with WireMock"""
    test_id = "connections.scim_configuration.get_default_mapping.0"
    client = get_client(test_id)
    client.connections.scim_configuration.get_default_mapping(
        id="id",
    )
    verify_request_count(test_id, "GET", "/connections/id/scim-configuration/default-mapping", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_webhook_basic_auth import EventStreamWebhookBasicAuth
from .event_stream_webhook_bearer_auth import EventStreamWebhookBearerAuth
from .event_stream_webhook_custom_header_auth import EventStreamWebhookCustomHeaderAuth

EventStreamWebhookAuthorizationResponse = typing.Union[
    EventStreamWebhookBasicAuth, EventStreamWebhookBearerAuth, EventStreamWebhookCustomHeaderAuth
]



# This file was auto-generated by Fern from our API Definition.

import typing

BreachedPasswordDetectionAdminNotificationFrequencyEnum = typing.Union[
    typing.Literal["immediately", "daily", "weekly", "monthly"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from .connection_waad_protocol_enum_azure_ad import ConnectionWaadProtocolEnumAzureAd

ConnectionWaadProtocol = ConnectionWaadProtocolEnumAzureAd
"""
The authentication protocol for Azure AD v1 endpoints. 'openid-connect' (default, recommended) uses modern OAuth 2.0/OIDC. 'ws-federation' is a legacy SAML-based protocol for older integrations. Only available with Azure AD v1 API.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionIpsAd = typing.List[str]
"""
Array of IP address ranges in CIDR notation used to determine if authentication requests originate from the corporate network for Kerberos or certificate authentication.
"""



# This file was auto-generated by Fern from our API Definition.

ConnectionGatewayAuthenticationAudienceSms = str
"""
`aud` claim value in the JWT sent to the SMS gateway. Identifies the gateway service (e.g., 'urn:MySmsGateway').
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.get_supplemental_signals_response_content import GetSupplementalSignalsResponseContent
from ..types.patch_supplemental_signals_response_content import PatchSupplementalSignalsResponseContent
from .raw_client import AsyncRawSupplementalSignalsClient, RawSupplementalSignalsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SupplementalSignalsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSupplementalSignalsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSupplementalSignalsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSupplementalSignalsClient
        """
        return self._raw_client

    def get(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetSupplementalSignalsResponseContent:
        """
        Get the supplemental signals configuration for a tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSupplementalSignalsResponseContent
            Supplemental Signals configuration successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.supplemental_signals.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def patch(
        self, *, akamai_enabled: bool, request_options: typing.Optional[RequestOptions] = None
    ) -> PatchSupplementalSignalsResponseContent:
        """
        Update the supplemental signals configuration for a tenant.

        Parameters
        ----------
        akamai_enabled : bool
            Indicates if incoming Akamai Headers should be processed

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchSupplementalSignalsResponseContent
            Supplemental Signals configuration successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.supplemental_signals.patch(
            akamai_enabled=True,
        )
        """
        _response = self._raw_client.patch(akamai_enabled=akamai_enabled, request_options=request_options)
        return _response.data


class AsyncSupplementalSignalsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSupplementalSignalsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSupplementalSignalsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSupplementalSignalsClient
        """
        return self._raw_client

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetSupplementalSignalsResponseContent:
        """
        Get the supplemental signals configuration for a tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSupplementalSignalsResponseContent
            Supplemental Signals configuration successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.supplemental_signals.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def patch(
        self, *, akamai_enabled: bool, request_options: typing.Optional[RequestOptions] = None
    ) -> PatchSupplementalSignalsResponseContent:
        """
        Update the supplemental signals configuration for a tenant.

        Parameters
        ----------
        akamai_enabled : bool
            Indicates if incoming Akamai Headers should be processed

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchSupplementalSignalsResponseContent
            Supplemental Signals configuration successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.supplemental_signals.patch(
                akamai_enabled=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch(akamai_enabled=akamai_enabled, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawVerificationClient, RawVerificationClient

if typing.TYPE_CHECKING:
    from .templates.client import AsyncTemplatesClient, TemplatesClient


class VerificationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVerificationClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._templates: typing.Optional[TemplatesClient] = None

    @property
    def with_raw_response(self) -> RawVerificationClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVerificationClient
        """
        return self._raw_client

    @property
    def templates(self):
        if self._templates is None:
            from .templates.client import TemplatesClient  # noqa: E402

            self._templates = TemplatesClient(client_wrapper=self._client_wrapper)
        return self._templates


class AsyncVerificationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVerificationClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._templates: typing.Optional[AsyncTemplatesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawVerificationClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVerificationClient
        """
        return self._raw_client

    @property
    def templates(self):
        if self._templates is None:
            from .templates.client import AsyncTemplatesClient  # noqa: E402

            self._templates = AsyncTemplatesClient(client_wrapper=self._client_wrapper)
        return self._templates



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionProfileOrganizationAssignMembershipOnLoginEnum = typing.Union[
    typing.Literal["none", "optional", "required"], typing.Any
]



"""Token Verifier module"""
from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .exceptions import TokenValidationError
from .rest_async import AsyncRestClient
from .token_verifier import AsymmetricSignatureVerifier, JwksFetcher, TokenVerifier

if TYPE_CHECKING:
    from aiohttp import ClientSession
    from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicKey


class AsyncAsymmetricSignatureVerifier(AsymmetricSignatureVerifier):
    """Async verifier for RSA signatures, which rely on public key certificates.

    Args:
        jwks_url (str): The url where the JWK set is located.
        algorithm (str, optional): The expected signing algorithm. Defaults to "RS256".
    """

    def __init__(self, jwks_url: str, algorithm: str = "RS256") -> None:
        super().__init__(jwks_url, algorithm)
        self._fetcher = AsyncJwksFetcher(jwks_url)

    def set_session(self, session: ClientSession) -> None:
        """Set Client Session to improve performance by reusing session.

        Args:
            session (aiohttp.ClientSession): The client session which should be closed
                manually or within context manager.
        """
        self._fetcher.set_session(session)

    async def _fetch_key(self, key_id=None):
        """Request the JWKS.

        Args:
        key_id (str): The key's key id."""
        return await self._fetcher.get_key(key_id)

    async def verify_signature(self, token) -> dict[str, Any]:
        """Verifies the signature of the given JSON web token.

        Args:
            token (str): The JWT to get its signature verified.

        Raises:
            TokenValidationError: if the token cannot be decoded, the algorithm is invalid
            or the token's signature doesn't match the calculated one.
        """
        kid = self._get_kid(token)
        secret_or_certificate = await self._fetch_key(key_id=kid)

        return self._decode_jwt(token, secret_or_certificate)


class AsyncJwksFetcher(JwksFetcher):
    """Class that async fetches and holds a JSON web key set.
    This class makes use of an in-memory cache. For it to work properly, define this instance once and re-use it.

    Args:
        jwks_url (str): The url where the JWK set is located.
        cache_ttl (str, optional): The lifetime of the JWK set cache in seconds. Defaults to 600 seconds.
    """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self._async_client = AsyncRestClient(None)

    def set_session(self, session: ClientSession) -> None:
        """Set Client Session to improve performance by reusing session.

        Args:
            session (aiohttp.ClientSession): The client session which should be closed
                manually or within context manager.
        """
        self._async_client.set_session(session)

    async def _fetch_jwks(self, force: bool = False) -> dict[str, RSAPublicKey]:
        """Attempts to obtain the JWK set from the cache, as long as it's still valid.
        When not, it will perform a network request to the jwks_url to obtain a fresh result
        and update the cache value with it.

        Args:
            force (bool, optional): whether to ignore the cache and force a network request or not. Defaults to False.
        """
        if force or self._cache_expired():
            self._cache_value = {}
            try:
                jwks = await self._async_client.get(self._jwks_url)
                self._cache_jwks(jwks)
            except:  # noqa: E722
                return self._cache_value
            return self._cache_value

        self._cache_is_fresh = False
        return self._cache_value

    async def get_key(self, key_id: str) -> RSAPublicKey:
        """Obtains the JWK associated with the given key id.

        Args:
            key_id (str): The id of the key to fetch.

        Returns:
            the JWK associated with the given key id.

        Raises:
            TokenValidationError: when a key with that id cannot be found
        """
        keys = await self._fetch_jwks()

        if keys and key_id in keys:
            return keys[key_id]

        if not self._cache_is_fresh:
            keys = await self._fetch_jwks(force=True)
            if keys and key_id in keys:
                return keys[key_id]
        raise TokenValidationError(f'RSA Public Key with ID "{key_id}" was not found.')


class AsyncTokenVerifier(TokenVerifier):
    """Class that verifies ID tokens following the steps defined in the OpenID Connect spec.
    An OpenID Connect ID token is not meant to be consumed until it's verified.

    Args:
        signature_verifier (AsyncAsymmetricSignatureVerifier): The instance that knows how to verify the signature.
        issuer (str): The expected issuer claim value.
        audience (str): The expected audience claim value.
        leeway (int, optional): The clock skew to accept when verifying date related claims in seconds.
        Defaults to 60 seconds.
    """

    def __init__(
        self,
        signature_verifier: AsyncAsymmetricSignatureVerifier,
        issuer: str,
        audience: str,
        leeway: int = 0,
    ) -> None:
        if not signature_verifier or not isinstance(
            signature_verifier, AsyncAsymmetricSignatureVerifier
        ):
            raise TypeError(
                "signature_verifier must be an instance of AsyncAsymmetricSignatureVerifier."
            )

        self.iss = issuer
        self.aud = audience
        self.leeway = leeway
        self._sv = signature_verifier
        self._clock = None  # legacy testing requirement

    def set_session(self, session: ClientSession) -> None:
        """Set Client Session to improve performance by reusing session.

        Args:
            session (aiohttp.ClientSession): The client session which should be closed
                manually or within context manager.
        """
        self._sv.set_session(session)

    async def verify(
        self,
        token: str,
        nonce: str | None = None,
        max_age: int | None = None,
        organization: str | None = None,
    ) -> dict[str, Any]:
        """Attempts to verify the given ID token, following the steps defined in the OpenID Connect spec.

        Args:
            token (str): The JWT to verify.
            nonce (str, optional): The nonce value sent during authentication.
            max_age (int, optional): The max_age value sent during authentication.
            organization (str, optional): The expected organization ID (org_id) or organization name (org_name) claim value. This should be specified
            when logging in to an organization.

        Returns:
            the decoded payload from the token

        Raises:
            TokenValidationError: when the token cannot be decoded, the token signing algorithm is not the expected one,
            the token signature is invalid or the token has a claim missing or with unexpected value.
        """

        # Verify token presence
        if not token or not isinstance(token, str):
            raise TokenValidationError("ID token is required but missing.")

        # Verify algorithm and signature
        payload = await self._sv.verify_signature(token)

        # Verify claims
        self._verify_payload(payload, nonce, max_age, organization)

        return payload



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_acr_values_supported import ConnectionAcrValuesSupported
from .connection_authorization_endpoint import ConnectionAuthorizationEndpoint
from .connection_claim_types_supported import ConnectionClaimTypesSupported
from .connection_claims_locales_supported import ConnectionClaimsLocalesSupported
from .connection_claims_parameter_supported import ConnectionClaimsParameterSupported
from .connection_claims_supported import ConnectionClaimsSupported
from .connection_display_values_supported import ConnectionDisplayValuesSupported
from .connection_dpop_signing_alg_values_supported import ConnectionDpopSigningAlgValuesSupported
from .connection_end_session_endpoint import ConnectionEndSessionEndpoint
from .connection_grant_types_supported import ConnectionGrantTypesSupported
from .connection_id_token_encryption_alg_values_supported import ConnectionIdTokenEncryptionAlgValuesSupported
from .connection_id_token_encryption_enc_values_supported import ConnectionIdTokenEncryptionEncValuesSupported
from .connection_id_token_signing_alg_values_supported import ConnectionIdTokenSigningAlgValuesSupported
from .connection_issuer import ConnectionIssuer
from .connection_jwks_uri import ConnectionJwksUri
from .connection_op_policy_uri import ConnectionOpPolicyUri
from .connection_op_tos_uri import ConnectionOpTosUri
from .connection_registration_endpoint import ConnectionRegistrationEndpoint
from .connection_request_object_encryption_alg_values_supported import (
    ConnectionRequestObjectEncryptionAlgValuesSupported,
)
from .connection_request_object_encryption_enc_values_supported import (
    ConnectionRequestObjectEncryptionEncValuesSupported,
)
from .connection_request_object_signing_alg_values_supported import ConnectionRequestObjectSigningAlgValuesSupported
from .connection_request_parameter_supported import ConnectionRequestParameterSupported
from .connection_request_uri_parameter_supported import ConnectionRequestUriParameterSupported
from .connection_require_request_uri_registration import ConnectionRequireRequestUriRegistration
from .connection_response_modes_supported import ConnectionResponseModesSupported
from .connection_response_types_supported import ConnectionResponseTypesSupported
from .connection_scopes_supported import ConnectionScopesSupported
from .connection_service_documentation import ConnectionServiceDocumentation
from .connection_subject_types_supported import ConnectionSubjectTypesSupported
from .connection_token_endpoint import ConnectionTokenEndpoint
from .connection_token_endpoint_auth_methods_supported import ConnectionTokenEndpointAuthMethodsSupported
from .connection_token_endpoint_auth_signing_alg_values_supported import (
    ConnectionTokenEndpointAuthSigningAlgValuesSupported,
)
from .connection_ui_locales_supported import ConnectionUiLocalesSupported
from .connection_userinfo_encryption_alg_values_supported import ConnectionUserinfoEncryptionAlgValuesSupported
from .connection_userinfo_encryption_enc_values_supported import ConnectionUserinfoEncryptionEncValuesSupported
from .connection_userinfo_endpoint import ConnectionUserinfoEndpoint
from .connection_userinfo_signing_alg_values_supported import ConnectionUserinfoSigningAlgValuesSupported


class ConnectionOptionsOidcMetadata(UniversalBaseModel):
    """
    OpenID Connect Provider Metadata as per https://openid.net/specs/openid-connect-discovery-1_0.html#ProviderMetadata
    """

    acr_values_supported: typing.Optional[ConnectionAcrValuesSupported] = None
    authorization_endpoint: ConnectionAuthorizationEndpoint
    claim_types_supported: typing.Optional[ConnectionClaimTypesSupported] = None
    claims_locales_supported: typing.Optional[ConnectionClaimsLocalesSupported] = None
    claims_parameter_supported: typing.Optional[ConnectionClaimsParameterSupported] = None
    claims_supported: typing.Optional[ConnectionClaimsSupported] = None
    display_values_supported: typing.Optional[ConnectionDisplayValuesSupported] = None
    dpop_signing_alg_values_supported: typing.Optional[ConnectionDpopSigningAlgValuesSupported] = None
    end_session_endpoint: typing.Optional[ConnectionEndSessionEndpoint] = None
    grant_types_supported: typing.Optional[ConnectionGrantTypesSupported] = None
    id_token_encryption_alg_values_supported: typing.Optional[ConnectionIdTokenEncryptionAlgValuesSupported] = None
    id_token_encryption_enc_values_supported: typing.Optional[ConnectionIdTokenEncryptionEncValuesSupported] = None
    id_token_signing_alg_values_supported: ConnectionIdTokenSigningAlgValuesSupported
    issuer: ConnectionIssuer
    jwks_uri: ConnectionJwksUri
    op_policy_uri: typing.Optional[ConnectionOpPolicyUri] = None
    op_tos_uri: typing.Optional[ConnectionOpTosUri] = None
    registration_endpoint: typing.Optional[ConnectionRegistrationEndpoint] = None
    request_object_encryption_alg_values_supported: typing.Optional[
        ConnectionRequestObjectEncryptionAlgValuesSupported
    ] = None
    request_object_encryption_enc_values_supported: typing.Optional[
        ConnectionRequestObjectEncryptionEncValuesSupported
    ] = None
    request_object_signing_alg_values_supported: typing.Optional[ConnectionRequestObjectSigningAlgValuesSupported] = (
        None
    )
    request_parameter_supported: typing.Optional[ConnectionRequestParameterSupported] = None
    request_uri_parameter_supported: typing.Optional[ConnectionRequestUriParameterSupported] = None
    require_request_uri_registration: typing.Optional[ConnectionRequireRequestUriRegistration] = None
    response_modes_supported: typing.Optional[ConnectionResponseModesSupported] = None
    response_types_supported: typing.Optional[ConnectionResponseTypesSupported] = None
    scopes_supported: typing.Optional[ConnectionScopesSupported] = None
    service_documentation: typing.Optional[ConnectionServiceDocumentation] = None
    subject_types_supported: typing.Optional[ConnectionSubjectTypesSupported] = None
    token_endpoint: typing.Optional[ConnectionTokenEndpoint] = None
    token_endpoint_auth_methods_supported: typing.Optional[ConnectionTokenEndpointAuthMethodsSupported] = None
    token_endpoint_auth_signing_alg_values_supported: typing.Optional[
        ConnectionTokenEndpointAuthSigningAlgValuesSupported
    ] = None
    ui_locales_supported: typing.Optional[ConnectionUiLocalesSupported] = None
    userinfo_encryption_alg_values_supported: typing.Optional[ConnectionUserinfoEncryptionAlgValuesSupported] = None
    userinfo_encryption_enc_values_supported: typing.Optional[ConnectionUserinfoEncryptionEncValuesSupported] = None
    userinfo_endpoint: typing.Optional[ConnectionUserinfoEndpoint] = None
    userinfo_signing_alg_values_supported: typing.Optional[ConnectionUserinfoSigningAlgValuesSupported] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import encode_path_param
from ....core.parse_error import ParsingError
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ....errors.bad_request_error import BadRequestError
from ....errors.conflict_error import ConflictError
from ....errors.forbidden_error import ForbiddenError
from ....errors.not_found_error import NotFoundError
from ....errors.too_many_requests_error import TooManyRequestsError
from ....errors.unauthorized_error import UnauthorizedError
from ....types.create_directory_synchronization_response_content import CreateDirectorySynchronizationResponseContent
from pydantic import ValidationError


class RawSynchronizationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CreateDirectorySynchronizationResponseContent]:
        """
        Request an on-demand synchronization of the directory.

        Parameters
        ----------
        id : str
            The id of the connection to trigger synchronization for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateDirectorySynchronizationResponseContent]
            The directory synchronization was triggered. See <strong>Response Schemas</strong> for schema.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"connections/{encode_path_param(id)}/directory-provisioning/synchronizations",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateDirectorySynchronizationResponseContent,
                    parse_obj_as(
                        type_=CreateDirectorySynchronizationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSynchronizationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CreateDirectorySynchronizationResponseContent]:
        """
        Request an on-demand synchronization of the directory.

        Parameters
        ----------
        id : str
            The id of the connection to trigger synchronization for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateDirectorySynchronizationResponseContent]
            The directory synchronization was triggered. See <strong>Response Schemas</strong> for schema.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"connections/{encode_path_param(id)}/directory-provisioning/synchronizations",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateDirectorySynchronizationResponseContent,
                    parse_obj_as(
                        type_=CreateDirectorySynchronizationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.connection_for_list import ConnectionForList
from ...types.connection_strategy_enum import ConnectionStrategyEnum
from ...types.list_client_connections_response_content import ListClientConnectionsResponseContent
from .raw_client import AsyncRawConnectionsClient, RawConnectionsClient


class ConnectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawConnectionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawConnectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawConnectionsClient
        """
        return self._raw_client

    def get(
        self,
        id: str,
        *,
        strategy: typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ConnectionForList, ListClientConnectionsResponseContent]:
        """
        Retrieve all connections that are enabled for the specified [Application](https://www.auth0.com/docs/get-started/applications), using checkpoint pagination. A list of fields to include or exclude for each connection may also be specified.

        - This endpoint requires the `read:connections` scope and any one of `read:clients` or `read:client_summary`.
        - **Note**: The first time you call this endpoint, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no further results are remaining.

        Parameters
        ----------
        id : str
            ID of the client for which to retrieve enabled connections.

        strategy : typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]]
            Provide strategies to only retrieve connections with such strategies

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ConnectionForList, ListClientConnectionsResponseContent]
            Success

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.clients.connections.get(
            id="id",
            strategy=["ad"],
            from_="from",
            take=1,
            fields="fields",
            include_fields=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.get(
            id,
            strategy=strategy,
            from_=from_,
            take=take,
            fields=fields,
            include_fields=include_fields,
            request_options=request_options,
        )


class AsyncConnectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawConnectionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawConnectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawConnectionsClient
        """
        return self._raw_client

    async def get(
        self,
        id: str,
        *,
        strategy: typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]] = None,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ConnectionForList, ListClientConnectionsResponseContent]:
        """
        Retrieve all connections that are enabled for the specified [Application](https://www.auth0.com/docs/get-started/applications), using checkpoint pagination. A list of fields to include or exclude for each connection may also be specified.

        - This endpoint requires the `read:connections` scope and any one of `read:clients` or `read:client_summary`.
        - **Note**: The first time you call this endpoint, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no further results are remaining.

        Parameters
        ----------
        id : str
            ID of the client for which to retrieve enabled connections.

        strategy : typing.Optional[typing.Union[ConnectionStrategyEnum, typing.Sequence[ConnectionStrategyEnum]]]
            Provide strategies to only retrieve connections with such strategies

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        fields : typing.Optional[str]
            A comma separated list of fields to include or exclude (depending on include_fields) from the result, empty to retrieve all fields

        include_fields : typing.Optional[bool]
            <code>true</code> if the fields specified are to be included in the result, <code>false</code> otherwise (defaults to <code>true</code>)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ConnectionForList, ListClientConnectionsResponseContent]
            Success

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.clients.connections.get(
                id="id",
                strategy=["ad"],
                from_="from",
                take=1,
                fields="fields",
                include_fields=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.get(
            id,
            strategy=strategy,
            from_=from_,
            take=take,
            fields=fields,
            include_fields=include_fields,
            request_options=request_options,
        )



import asyncio
import threading
import time
from typing import Optional

import httpx

LEEWAY_SECONDS = 10


class TokenProvider:
    """
    Sync token provider with caching and automatic refresh.

    Fetches tokens via OAuth 2.0 client credentials grant and caches them
    until they expire (with a 10-second leeway for safety).

    Thread-safe: uses a lock to prevent concurrent token fetches.
    """

    def __init__(
        self,
        *,
        domain: str,
        client_id: str,
        client_secret: str,
        audience: Optional[str] = None,
    ):
        self._domain = domain
        self._client_id = client_id
        self._client_secret = client_secret
        self._audience = audience or f"https://{domain}/api/v2/"

        self._access_token: Optional[str] = None
        self._expires_at: float = 0
        self._lock = threading.Lock()

    def get_token(self) -> str:
        """
        Get a valid access token, fetching a new one if expired.

        Returns
        -------
        str
            A valid access token.
        """
        # Fast path: return cached token if still valid
        if self._access_token and time.time() < (self._expires_at - LEEWAY_SECONDS):
            return self._access_token

        with self._lock:
            # Double-check after acquiring lock
            if self._access_token and time.time() < (self._expires_at - LEEWAY_SECONDS):
                return self._access_token
            self._fetch_token()

        return self._access_token  # type: ignore[return-value]

    def _fetch_token(self) -> None:
        """Fetch a new token via client credentials grant."""
        url = f"https://{self._domain}/oauth/token"
        payload = {
            "grant_type": "client_credentials",
            "client_id": self._client_id,
            "client_secret": self._client_secret,
            "audience": self._audience,
        }

        with httpx.Client() as client:
            response = client.post(url, data=payload)
            response.raise_for_status()
            data = response.json()

        self._access_token = data["access_token"]
        self._expires_at = time.time() + data["expires_in"]


class AsyncTokenProvider:
    """
    Async token provider with caching and automatic refresh.

    Fetches tokens via OAuth 2.0 client credentials grant and caches them
    until they expire (with a 10-second leeway for safety).

    Coroutine-safe: uses an async lock to prevent concurrent token fetches.
    """

    def __init__(
        self,
        *,
        domain: str,
        client_id: str,
        client_secret: str,
        audience: Optional[str] = None,
    ):
        self._domain = domain
        self._client_id = client_id
        self._client_secret = client_secret
        self._audience = audience or f"https://{domain}/api/v2/"

        self._access_token: Optional[str] = None
        self._expires_at: float = 0
        self._lock = asyncio.Lock()

    async def get_token(self) -> str:
        """
        Get a valid access token, fetching a new one if expired.

        Returns
        -------
        str
            A valid access token.
        """
        # Fast path: return cached token if still valid
        if self._access_token and time.time() < (self._expires_at - LEEWAY_SECONDS):
            return self._access_token

        async with self._lock:
            # Double-check after acquiring lock
            if self._access_token and time.time() < (self._expires_at - LEEWAY_SECONDS):
                return self._access_token
            await self._fetch_token()

        return self._access_token  # type: ignore[return-value]

    async def _fetch_token(self) -> None:
        """Fetch a new token via client credentials grant."""
        url = f"https://{self._domain}/oauth/token"
        payload = {
            "grant_type": "client_credentials",
            "client_id": self._client_id,
            "client_secret": self._client_secret,
            "audience": self._audience,
        }

        async with httpx.AsyncClient() as client:
            response = await client.post(url, data=payload)
            response.raise_for_status()
            data = response.json()

        self._access_token = data["access_token"]
        self._expires_at = time.time() + data["expires_in"]



# This file was auto-generated by Fern from our API Definition.

import typing

from .client_signing_key import ClientSigningKey

ClientSigningKeys = typing.Optional[typing.List[ClientSigningKey]]
"""
Signing certificates associated with this client.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .device_credential_type_enum import DeviceCredentialTypeEnum


class DeviceCredential(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of this device.
    """

    device_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    User agent for this device
    """

    device_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the device. NOTE: This field is generally not populated for refresh_tokens and rotating_refresh_tokens
    """

    type: typing.Optional[DeviceCredentialTypeEnum] = pydantic.Field(default=None)
    """
    Type of credential. Can be `public_key`, `refresh_token`, or `rotating_refresh_token`.
    """

    user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    user_id this credential is associated with.
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    client_id of the client (application) this credential is for.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action_module_list_item import ActionModuleListItem


class GetActionModulesResponseContent(UniversalBaseModel):
    modules: typing.Optional[typing.List[ActionModuleListItem]] = pydantic.Field(default=None)
    """
    A list of ActionsModule objects.
    """

    total: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total number of modules in the tenant.
    """

    page: typing.Optional[int] = pydantic.Field(default=None)
    """
    The page index of the returned results.
    """

    per_page: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of results requested per page.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .csp_report_to_endpoint import CspReportToEndpoint

CspReportToEndpoints = typing.List[CspReportToEndpoint]
"""
Array of reporting endpoints.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .flows_vault_connectio_setup_api_key import FlowsVaultConnectioSetupApiKey
from .flows_vault_connectio_setup_api_key_with_base_url import FlowsVaultConnectioSetupApiKeyWithBaseUrl
from .flows_vault_connectio_setup_bigquery_oauth_jwt import FlowsVaultConnectioSetupBigqueryOauthJwt
from .flows_vault_connectio_setup_http_bearer import FlowsVaultConnectioSetupHttpBearer
from .flows_vault_connectio_setup_jwt import FlowsVaultConnectioSetupJwt
from .flows_vault_connectio_setup_mailjet_api_key import FlowsVaultConnectioSetupMailjetApiKey
from .flows_vault_connectio_setup_oauth_app import FlowsVaultConnectioSetupOauthApp
from .flows_vault_connectio_setup_oauth_code import FlowsVaultConnectioSetupOauthCode
from .flows_vault_connectio_setup_secret_api_key import FlowsVaultConnectioSetupSecretApiKey
from .flows_vault_connectio_setup_stripe_key_pair import FlowsVaultConnectioSetupStripeKeyPair
from .flows_vault_connectio_setup_token import FlowsVaultConnectioSetupToken
from .flows_vault_connectio_setup_twilio_api_key import FlowsVaultConnectioSetupTwilioApiKey
from .flows_vault_connectio_setup_webhook import FlowsVaultConnectioSetupWebhook
from .flows_vault_connection_http_api_key_setup import FlowsVaultConnectionHttpApiKeySetup
from .flows_vault_connection_http_basic_auth_setup import FlowsVaultConnectionHttpBasicAuthSetup
from .flows_vault_connection_http_oauth_client_credentials_setup import (
    FlowsVaultConnectionHttpOauthClientCredentialsSetup,
)

UpdateFlowsVaultConnectionSetup = typing.Union[
    FlowsVaultConnectioSetupApiKeyWithBaseUrl,
    FlowsVaultConnectioSetupApiKey,
    FlowsVaultConnectioSetupOauthApp,
    FlowsVaultConnectioSetupBigqueryOauthJwt,
    FlowsVaultConnectioSetupSecretApiKey,
    FlowsVaultConnectioSetupHttpBearer,
    FlowsVaultConnectionHttpBasicAuthSetup,
    FlowsVaultConnectionHttpApiKeySetup,
    FlowsVaultConnectionHttpOauthClientCredentialsSetup,
    FlowsVaultConnectioSetupJwt,
    FlowsVaultConnectioSetupMailjetApiKey,
    FlowsVaultConnectioSetupToken,
    FlowsVaultConnectioSetupWebhook,
    FlowsVaultConnectioSetupStripeKeyPair,
    FlowsVaultConnectioSetupOauthCode,
    FlowsVaultConnectioSetupTwilioApiKey,
]



# This file was auto-generated by Fern from our API Definition.

ConnectionClientIdLine = str
"""
LINE Channel ID issued by LINE during application registration. This value identifies your Auth0 connection to LINE.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class EmailProviderCredentials(UniversalBaseModel):
    """
    Credentials required to use the provider.
    """

    api_user: typing.Optional[str] = pydantic.Field(default=None)
    """
    API User.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    AWS or SparkPost region.
    """

    smtp_host: typing.Optional[str] = pydantic.Field(default=None)
    """
    SMTP host.
    """

    smtp_port: typing.Optional[int] = pydantic.Field(default=None)
    """
    SMTP port.
    """

    smtp_user: typing.Optional[str] = pydantic.Field(default=None)
    """
    SMTP username.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from typing import Any

from .base import AuthenticationBase


class Enterprise(AuthenticationBase):

    """Enterprise endpoints.

    Args:
        domain (str): Your auth0 domain (e.g: my-domain.us.auth0.com)
    """

    def saml_metadata(self) -> Any:
        """Get SAML2.0 Metadata."""

        return self.get(
            url="{}://{}/samlp/metadata/{}".format(
                self.protocol, self.domain, self.client_id
            )
        )

    def wsfed_metadata(self) -> Any:
        """Returns the WS-Federation Metadata."""

        url = "{}://{}/wsfed/FederationMetadata/2007-06/FederationMetadata.xml"

        return self.get(url=url.format(self.protocol, self.domain))



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CreateImportUsersResponseContent(UniversalBaseModel):
    status: str = pydantic.Field()
    """
    Status of this job.
    """

    type: str = pydantic.Field()
    """
    Type of job this is.
    """

    created_at: str = pydantic.Field()
    """
    When this job was created.
    """

    id: str = pydantic.Field()
    """
    ID of this job.
    """

    connection_id: str = pydantic.Field()
    """
    connection_id of the connection to which users will be imported.
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Customer-defined ID.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.guardian_factor import GuardianFactor
from ...types.guardian_factor_name_enum import GuardianFactorNameEnum
from ...types.set_guardian_factor_response_content import SetGuardianFactorResponseContent
from .raw_client import AsyncRawFactorsClient, RawFactorsClient

if typing.TYPE_CHECKING:
    from .duo.client import AsyncDuoClient, DuoClient
    from .phone.client import AsyncPhoneClient, PhoneClient
    from .push_notification.client import AsyncPushNotificationClient, PushNotificationClient
    from .sms.client import AsyncSmsClient, SmsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FactorsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFactorsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._phone: typing.Optional[PhoneClient] = None
        self._push_notification: typing.Optional[PushNotificationClient] = None
        self._sms: typing.Optional[SmsClient] = None
        self._duo: typing.Optional[DuoClient] = None

    @property
    def with_raw_response(self) -> RawFactorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFactorsClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[GuardianFactor]:
        """
        Retrieve details of all <a href="https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors">multi-factor authentication factors</a> associated with your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[GuardianFactor]
            Factors successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.guardian.factors.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def set(
        self, name: GuardianFactorNameEnum, *, enabled: bool, request_options: typing.Optional[RequestOptions] = None
    ) -> SetGuardianFactorResponseContent:
        """
        Update the status (i.e., enabled or disabled) of a specific multi-factor authentication factor.

        Parameters
        ----------
        name : GuardianFactorNameEnum
            Factor name. Can be `sms`, `push-notification`, `email`, `duo` `otp` `webauthn-roaming`, `webauthn-platform`, or `recovery-code`.

        enabled : bool
            Whether this factor is enabled (true) or disabled (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetGuardianFactorResponseContent
            Factor updated successfully.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.guardian.factors.set(
            name="push-notification",
            enabled=True,
        )
        """
        _response = self._raw_client.set(name, enabled=enabled, request_options=request_options)
        return _response.data

    @property
    def phone(self):
        if self._phone is None:
            from .phone.client import PhoneClient  # noqa: E402

            self._phone = PhoneClient(client_wrapper=self._client_wrapper)
        return self._phone

    @property
    def push_notification(self):
        if self._push_notification is None:
            from .push_notification.client import PushNotificationClient  # noqa: E402

            self._push_notification = PushNotificationClient(client_wrapper=self._client_wrapper)
        return self._push_notification

    @property
    def sms(self):
        if self._sms is None:
            from .sms.client import SmsClient  # noqa: E402

            self._sms = SmsClient(client_wrapper=self._client_wrapper)
        return self._sms

    @property
    def duo(self):
        if self._duo is None:
            from .duo.client import DuoClient  # noqa: E402

            self._duo = DuoClient(client_wrapper=self._client_wrapper)
        return self._duo


class AsyncFactorsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFactorsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._phone: typing.Optional[AsyncPhoneClient] = None
        self._push_notification: typing.Optional[AsyncPushNotificationClient] = None
        self._sms: typing.Optional[AsyncSmsClient] = None
        self._duo: typing.Optional[AsyncDuoClient] = None

    @property
    def with_raw_response(self) -> AsyncRawFactorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFactorsClient
        """
        return self._raw_client

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[GuardianFactor]:
        """
        Retrieve details of all <a href="https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors">multi-factor authentication factors</a> associated with your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[GuardianFactor]
            Factors successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.guardian.factors.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def set(
        self, name: GuardianFactorNameEnum, *, enabled: bool, request_options: typing.Optional[RequestOptions] = None
    ) -> SetGuardianFactorResponseContent:
        """
        Update the status (i.e., enabled or disabled) of a specific multi-factor authentication factor.

        Parameters
        ----------
        name : GuardianFactorNameEnum
            Factor name. Can be `sms`, `push-notification`, `email`, `duo` `otp` `webauthn-roaming`, `webauthn-platform`, or `recovery-code`.

        enabled : bool
            Whether this factor is enabled (true) or disabled (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetGuardianFactorResponseContent
            Factor updated successfully.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.guardian.factors.set(
                name="push-notification",
                enabled=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set(name, enabled=enabled, request_options=request_options)
        return _response.data

    @property
    def phone(self):
        if self._phone is None:
            from .phone.client import AsyncPhoneClient  # noqa: E402

            self._phone = AsyncPhoneClient(client_wrapper=self._client_wrapper)
        return self._phone

    @property
    def push_notification(self):
        if self._push_notification is None:
            from .push_notification.client import AsyncPushNotificationClient  # noqa: E402

            self._push_notification = AsyncPushNotificationClient(client_wrapper=self._client_wrapper)
        return self._push_notification

    @property
    def sms(self):
        if self._sms is None:
            from .sms.client import AsyncSmsClient  # noqa: E402

            self._sms = AsyncSmsClient(client_wrapper=self._client_wrapper)
        return self._sms

    @property
    def duo(self):
        if self._duo is None:
            from .duo.client import AsyncDuoClient  # noqa: E402

            self._duo = AsyncDuoClient(client_wrapper=self._client_wrapper)
        return self._duo



# This file was auto-generated by Fern from our API Definition.

import typing

OauthScope = typing.Union[
    typing.Literal[
        "create:actions",
        "read:actions",
        "update:actions",
        "delete:actions",
        "read:anomaly_blocks",
        "delete:anomaly_blocks",
        "read:attack_protection",
        "update:attack_protection",
        "create:authentication_methods",
        "read:authentication_methods",
        "update:authentication_methods",
        "delete:authentication_methods",
        "read:branding",
        "update:branding",
        "delete:branding",
        "create:client_credentials",
        "read:client_credentials",
        "update:client_credentials",
        "delete:client_credentials",
        "create:client_grants",
        "read:client_grants",
        "update:client_grants",
        "delete:client_grants",
        "read:client_keys",
        "update:client_keys",
        "read:client_summary",
        "update:client_token_vault_privileged_access",
        "create:clients",
        "read:clients",
        "update:clients",
        "delete:clients",
        "create:connection_profiles",
        "read:connection_profiles",
        "update:connection_profiles",
        "delete:connection_profiles",
        "create:connections",
        "read:connections",
        "update:connections",
        "delete:connections",
        "create:connections_keys",
        "read:connections_keys",
        "update:connections_keys",
        "read:current_user",
        "delete:current_user",
        "create:current_user_device_credentials",
        "delete:current_user_device_credentials",
        "update:current_user_identities",
        "update:current_user_metadata",
        "create:custom_domains",
        "read:custom_domains",
        "update:custom_domains",
        "delete:custom_domains",
        "create:custom_signing_keys",
        "read:custom_signing_keys",
        "update:custom_signing_keys",
        "delete:custom_signing_keys",
        "read:device_credentials",
        "delete:device_credentials",
        "create:directory_provisionings",
        "read:directory_provisionings",
        "update:directory_provisionings",
        "delete:directory_provisionings",
        "create:email_provider",
        "read:email_provider",
        "update:email_provider",
        "delete:email_provider",
        "create:email_templates",
        "read:email_templates",
        "update:email_templates",
        "create:encryption_keys",
        "read:encryption_keys",
        "update:encryption_keys",
        "delete:encryption_keys",
        "read:event_deliveries",
        "update:event_deliveries",
        "create:event_streams",
        "read:event_streams",
        "update:event_streams",
        "delete:event_streams",
        "read:events",
        "create:flows",
        "read:flows",
        "update:flows",
        "delete:flows",
        "read:flows_executions",
        "delete:flows_executions",
        "create:flows_vault_connections",
        "read:flows_vault_connections",
        "update:flows_vault_connections",
        "delete:flows_vault_connections",
        "create:forms",
        "read:forms",
        "update:forms",
        "delete:forms",
        "read:grants",
        "delete:grants",
        "read:group_members",
        "create:group_roles",
        "read:group_roles",
        "delete:group_roles",
        "read:groups",
        "delete:groups",
        "create:guardian_enrollment_tickets",
        "read:guardian_enrollments",
        "delete:guardian_enrollments",
        "read:guardian_factors",
        "update:guardian_factors",
        "create:hooks",
        "read:hooks",
        "update:hooks",
        "delete:hooks",
        "create:log_streams",
        "read:log_streams",
        "update:log_streams",
        "delete:log_streams",
        "read:logs",
        "read:logs_users",
        "read:mfa_policies",
        "update:mfa_policies",
        "create:network_acls",
        "read:network_acls",
        "update:network_acls",
        "delete:network_acls",
        "create:organization_client_grants",
        "read:organization_client_grants",
        "delete:organization_client_grants",
        "create:organization_connections",
        "read:organization_connections",
        "update:organization_connections",
        "delete:organization_connections",
        "create:organization_discovery_domains",
        "read:organization_discovery_domains",
        "update:organization_discovery_domains",
        "delete:organization_discovery_domains",
        "create:organization_group_roles",
        "read:organization_group_roles",
        "delete:organization_group_roles",
        "read:organization_groups",
        "create:organization_invitations",
        "read:organization_invitations",
        "delete:organization_invitations",
        "read:organization_member_effective_roles",
        "read:organization_member_role_source_groups",
        "create:organization_member_roles",
        "read:organization_member_roles",
        "delete:organization_member_roles",
        "create:organization_members",
        "read:organization_members",
        "delete:organization_members",
        "create:organizations",
        "read:organizations",
        "update:organizations",
        "delete:organizations",
        "read:organizations_summary",
        "create:phone_providers",
        "read:phone_providers",
        "update:phone_providers",
        "delete:phone_providers",
        "create:phone_templates",
        "read:phone_templates",
        "update:phone_templates",
        "delete:phone_templates",
        "read:prompts",
        "update:prompts",
        "create:rate_limit_policies",
        "read:rate_limit_policies",
        "update:rate_limit_policies",
        "delete:rate_limit_policies",
        "read:refresh_tokens",
        "update:refresh_tokens",
        "delete:refresh_tokens",
        "create:resource_servers",
        "read:resource_servers",
        "update:resource_servers",
        "delete:resource_servers",
        "create:role_members",
        "read:role_members",
        "delete:role_members",
        "create:roles",
        "read:roles",
        "update:roles",
        "delete:roles",
        "create:rules",
        "read:rules",
        "update:rules",
        "delete:rules",
        "read:rules_configs",
        "update:rules_configs",
        "delete:rules_configs",
        "create:scim_config",
        "read:scim_config",
        "update:scim_config",
        "delete:scim_config",
        "create:scim_token",
        "read:scim_token",
        "delete:scim_token",
        "read:self_service_profile_custom_texts",
        "update:self_service_profile_custom_texts",
        "create:self_service_profiles",
        "read:self_service_profiles",
        "update:self_service_profiles",
        "delete:self_service_profiles",
        "read:sessions",
        "update:sessions",
        "delete:sessions",
        "create:signing_keys",
        "read:signing_keys",
        "update:signing_keys",
        "create:sso_access_tickets",
        "delete:sso_access_tickets",
        "read:stats",
        "read:tenant_settings",
        "update:tenant_settings",
        "create:token_exchange_profiles",
        "read:token_exchange_profiles",
        "update:token_exchange_profiles",
        "delete:token_exchange_profiles",
        "create:user_attribute_profiles",
        "read:user_attribute_profiles",
        "update:user_attribute_profiles",
        "delete:user_attribute_profiles",
        "read:user_effective_permissions",
        "read:user_effective_roles",
        "read:user_idp_tokens",
        "read:user_permission_source_roles",
        "read:user_role_source_groups",
        "create:user_tickets",
        "create:users",
        "read:users",
        "update:users",
        "delete:users",
        "update:users_app_metadata",
        "create:vdcs_templates",
        "read:vdcs_templates",
        "update:vdcs_templates",
        "delete:vdcs_templates",
    ],
    typing.Any,
]



from .conftest import get_client, verify_request_count


def test_roles_groups_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "roles.groups.get.0"
    client = get_client(test_id)
    client.roles.groups.get(
        id="id",
        from_="from",
        take=1,
    )
    verify_request_count(test_id, "GET", "/roles/id/groups", {"from": "from", "take": "1"}, 1)


def test_roles_groups_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "roles.groups.create.0"
    client = get_client(test_id)
    client.roles.groups.create(
        id="id",
        groups=["groups"],
    )
    verify_request_count(test_id, "POST", "/roles/id/groups", None, 1)


def test_roles_groups_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "roles.groups.delete.0"
    client = get_client(test_id)
    client.roles.groups.delete(
        id="id",
        groups=["groups"],
    )
    verify_request_count(test_id, "DELETE", "/roles/id/groups", None, 1)



# This file was auto-generated by Fern from our API Definition.

from .connection_scope_array import ConnectionScopeArray

ConnectionScopePaypal = ConnectionScopeArray
"""
OAuth 2.0 scopes requested from PayPal during authorization. Built automatically from the enabled attribute flags (profile, email, address, phone) plus any freeform_scopes. Always includes 'openid' as the base scope.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.create_rate_limit_policy_response_content import CreateRateLimitPolicyResponseContent
from ..types.get_rate_limit_policy_response_content import GetRateLimitPolicyResponseContent
from ..types.list_rate_limit_policies_paginated_response_content import ListRateLimitPoliciesPaginatedResponseContent
from ..types.patch_rate_limit_policy_configuration_request_content import (
    PatchRateLimitPolicyConfigurationRequestContent,
)
from ..types.rate_limit_policy import RateLimitPolicy
from ..types.rate_limit_policy_configuration import RateLimitPolicyConfiguration
from ..types.rate_limit_policy_consumer_enum import RateLimitPolicyConsumerEnum
from ..types.rate_limit_policy_resource_enum import RateLimitPolicyResourceEnum
from ..types.update_rate_limit_policy_response_content import UpdateRateLimitPolicyResponseContent
from .raw_client import AsyncRawRateLimitPoliciesClient, RawRateLimitPoliciesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RateLimitPoliciesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRateLimitPoliciesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRateLimitPoliciesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRateLimitPoliciesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        resource: typing.Optional[RateLimitPolicyResourceEnum] = None,
        consumer: typing.Optional[RateLimitPolicyConsumerEnum] = None,
        consumer_selector: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        from_: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[RateLimitPolicy, ListRateLimitPoliciesPaginatedResponseContent]:
        """
        Parameters
        ----------
        resource : typing.Optional[RateLimitPolicyResourceEnum]
            The API protected by the Rate Limit Policy.

        consumer : typing.Optional[RateLimitPolicyConsumerEnum]
            The consumer to which the rate limit policy applies.

        consumer_selector : typing.Optional[str]
            Identifier or category within the consumer to which the policy applies. Supported values: `client_id:<client_id>` to target a specific client by ID, `client_id:<cimd_uri>` to target a CIMD client by URI, `cimd_clients` to target all CIMD clients, `third_party_clients` to target all third-party clients, or `default` to apply the policy to any consumer identifier not otherwise explicitly targeted.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        from_ : typing.Optional[str]
            Cursor for pagination.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[RateLimitPolicy, ListRateLimitPoliciesPaginatedResponseContent]
            Rate limit policies retrieved successfully.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.rate_limit_policies.list(
            resource="oauth_authentication_api",
            consumer="client",
            consumer_selector="consumer_selector",
            take=1,
            from_="from",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            resource=resource,
            consumer=consumer,
            consumer_selector=consumer_selector,
            take=take,
            from_=from_,
            request_options=request_options,
        )

    def create(
        self,
        *,
        resource: RateLimitPolicyResourceEnum,
        consumer: RateLimitPolicyConsumerEnum,
        consumer_selector: str,
        configuration: RateLimitPolicyConfiguration,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateRateLimitPolicyResponseContent:
        """
        Parameters
        ----------
        resource : RateLimitPolicyResourceEnum

        consumer : RateLimitPolicyConsumerEnum

        consumer_selector : str
            Identifier or category within the consumer to which the policy applies. Supported values: `client_id:<client_id>` to target a specific client by ID, `client_id:<cimd_uri>` to target a CIMD client by URI, `cimd_clients` to target all CIMD clients, `third_party_clients` to target all third-party clients, or `default` to apply the policy to any consumer identifier not otherwise explicitly targeted.

        configuration : RateLimitPolicyConfiguration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateRateLimitPolicyResponseContent
            Rate limit policy successfully created.

        Examples
        --------
        from auth0 import Auth0, RateLimitPolicyConfigurationZero

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.rate_limit_policies.create(
            resource="oauth_authentication_api",
            consumer="client",
            consumer_selector="consumer_selector",
            configuration=RateLimitPolicyConfigurationZero(
                action="allow",
            ),
        )
        """
        _response = self._raw_client.create(
            resource=resource,
            consumer=consumer,
            consumer_selector=consumer_selector,
            configuration=configuration,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetRateLimitPolicyResponseContent:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetRateLimitPolicyResponseContent
            Rate limit policy retrieved successfully.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.rate_limit_policies.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.rate_limit_policies.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        configuration: PatchRateLimitPolicyConfigurationRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateRateLimitPolicyResponseContent:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        configuration : PatchRateLimitPolicyConfigurationRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateRateLimitPolicyResponseContent
            Rate limit policy successfully updated.

        Examples
        --------
        from auth0 import Auth0, PatchRateLimitPolicyConfigurationRequestContentZero

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.rate_limit_policies.update(
            id="id",
            configuration=PatchRateLimitPolicyConfigurationRequestContentZero(
                action="allow",
            ),
        )
        """
        _response = self._raw_client.update(id, configuration=configuration, request_options=request_options)
        return _response.data


class AsyncRateLimitPoliciesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRateLimitPoliciesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRateLimitPoliciesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRateLimitPoliciesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        resource: typing.Optional[RateLimitPolicyResourceEnum] = None,
        consumer: typing.Optional[RateLimitPolicyConsumerEnum] = None,
        consumer_selector: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        from_: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[RateLimitPolicy, ListRateLimitPoliciesPaginatedResponseContent]:
        """
        Parameters
        ----------
        resource : typing.Optional[RateLimitPolicyResourceEnum]
            The API protected by the Rate Limit Policy.

        consumer : typing.Optional[RateLimitPolicyConsumerEnum]
            The consumer to which the rate limit policy applies.

        consumer_selector : typing.Optional[str]
            Identifier or category within the consumer to which the policy applies. Supported values: `client_id:<client_id>` to target a specific client by ID, `client_id:<cimd_uri>` to target a CIMD client by URI, `cimd_clients` to target all CIMD clients, `third_party_clients` to target all third-party clients, or `default` to apply the policy to any consumer identifier not otherwise explicitly targeted.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        from_ : typing.Optional[str]
            Cursor for pagination.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[RateLimitPolicy, ListRateLimitPoliciesPaginatedResponseContent]
            Rate limit policies retrieved successfully.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.rate_limit_policies.list(
                resource="oauth_authentication_api",
                consumer="client",
                consumer_selector="consumer_selector",
                take=1,
                from_="from",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            resource=resource,
            consumer=consumer,
            consumer_selector=consumer_selector,
            take=take,
            from_=from_,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        resource: RateLimitPolicyResourceEnum,
        consumer: RateLimitPolicyConsumerEnum,
        consumer_selector: str,
        configuration: RateLimitPolicyConfiguration,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateRateLimitPolicyResponseContent:
        """
        Parameters
        ----------
        resource : RateLimitPolicyResourceEnum

        consumer : RateLimitPolicyConsumerEnum

        consumer_selector : str
            Identifier or category within the consumer to which the policy applies. Supported values: `client_id:<client_id>` to target a specific client by ID, `client_id:<cimd_uri>` to target a CIMD client by URI, `cimd_clients` to target all CIMD clients, `third_party_clients` to target all third-party clients, or `default` to apply the policy to any consumer identifier not otherwise explicitly targeted.

        configuration : RateLimitPolicyConfiguration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateRateLimitPolicyResponseContent
            Rate limit policy successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, RateLimitPolicyConfigurationZero

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.rate_limit_policies.create(
                resource="oauth_authentication_api",
                consumer="client",
                consumer_selector="consumer_selector",
                configuration=RateLimitPolicyConfigurationZero(
                    action="allow",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            resource=resource,
            consumer=consumer,
            consumer_selector=consumer_selector,
            configuration=configuration,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetRateLimitPolicyResponseContent:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetRateLimitPolicyResponseContent
            Rate limit policy retrieved successfully.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.rate_limit_policies.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.rate_limit_policies.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        configuration: PatchRateLimitPolicyConfigurationRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateRateLimitPolicyResponseContent:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        configuration : PatchRateLimitPolicyConfigurationRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateRateLimitPolicyResponseContent
            Rate limit policy successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import (
            AsyncAuth0,
            PatchRateLimitPolicyConfigurationRequestContentZero,
        )

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.rate_limit_policies.update(
                id="id",
                configuration=PatchRateLimitPolicyConfigurationRequestContentZero(
                    action="allow",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, configuration=configuration, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.create_rate_limit_policy_response_content import CreateRateLimitPolicyResponseContent
from ..types.get_rate_limit_policy_response_content import GetRateLimitPolicyResponseContent
from ..types.list_rate_limit_policies_paginated_response_content import ListRateLimitPoliciesPaginatedResponseContent
from ..types.patch_rate_limit_policy_configuration_request_content import (
    PatchRateLimitPolicyConfigurationRequestContent,
)
from ..types.rate_limit_policy import RateLimitPolicy
from ..types.rate_limit_policy_configuration import RateLimitPolicyConfiguration
from ..types.rate_limit_policy_consumer_enum import RateLimitPolicyConsumerEnum
from ..types.rate_limit_policy_resource_enum import RateLimitPolicyResourceEnum
from ..types.update_rate_limit_policy_response_content import UpdateRateLimitPolicyResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRateLimitPoliciesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        resource: typing.Optional[RateLimitPolicyResourceEnum] = None,
        consumer: typing.Optional[RateLimitPolicyConsumerEnum] = None,
        consumer_selector: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        from_: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[RateLimitPolicy, ListRateLimitPoliciesPaginatedResponseContent]:
        """
        Parameters
        ----------
        resource : typing.Optional[RateLimitPolicyResourceEnum]
            The API protected by the Rate Limit Policy.

        consumer : typing.Optional[RateLimitPolicyConsumerEnum]
            The consumer to which the rate limit policy applies.

        consumer_selector : typing.Optional[str]
            Identifier or category within the consumer to which the policy applies. Supported values: `client_id:<client_id>` to target a specific client by ID, `client_id:<cimd_uri>` to target a CIMD client by URI, `cimd_clients` to target all CIMD clients, `third_party_clients` to target all third-party clients, or `default` to apply the policy to any consumer identifier not otherwise explicitly targeted.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        from_ : typing.Optional[str]
            Cursor for pagination.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[RateLimitPolicy, ListRateLimitPoliciesPaginatedResponseContent]
            Rate limit policies retrieved successfully.
        """
        _response = self._client_wrapper.httpx_client.request(
            "rate-limit-policies",
            method="GET",
            params={
                "resource": resource,
                "consumer": consumer,
                "consumer_selector": consumer_selector,
                "take": take,
                "from": from_,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListRateLimitPoliciesPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListRateLimitPoliciesPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.rate_limit_policies
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    resource=resource,
                    consumer=consumer,
                    consumer_selector=consumer_selector,
                    take=take,
                    from_=_parsed_next,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        resource: RateLimitPolicyResourceEnum,
        consumer: RateLimitPolicyConsumerEnum,
        consumer_selector: str,
        configuration: RateLimitPolicyConfiguration,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateRateLimitPolicyResponseContent]:
        """
        Parameters
        ----------
        resource : RateLimitPolicyResourceEnum

        consumer : RateLimitPolicyConsumerEnum

        consumer_selector : str
            Identifier or category within the consumer to which the policy applies. Supported values: `client_id:<client_id>` to target a specific client by ID, `client_id:<cimd_uri>` to target a CIMD client by URI, `cimd_clients` to target all CIMD clients, `third_party_clients` to target all third-party clients, or `default` to apply the policy to any consumer identifier not otherwise explicitly targeted.

        configuration : RateLimitPolicyConfiguration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateRateLimitPolicyResponseContent]
            Rate limit policy successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "rate-limit-policies",
            method="POST",
            json={
                "resource": resource,
                "consumer": consumer,
                "consumer_selector": consumer_selector,
                "configuration": convert_and_respect_annotation_metadata(
                    object_=configuration, annotation=RateLimitPolicyConfiguration, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateRateLimitPolicyResponseContent,
                    parse_obj_as(
                        type_=CreateRateLimitPolicyResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetRateLimitPolicyResponseContent]:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetRateLimitPolicyResponseContent]
            Rate limit policy retrieved successfully.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"rate-limit-policies/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetRateLimitPolicyResponseContent,
                    parse_obj_as(
                        type_=GetRateLimitPolicyResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"rate-limit-policies/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        configuration: PatchRateLimitPolicyConfigurationRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateRateLimitPolicyResponseContent]:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        configuration : PatchRateLimitPolicyConfigurationRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateRateLimitPolicyResponseContent]
            Rate limit policy successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"rate-limit-policies/{encode_path_param(id)}",
            method="PATCH",
            json={
                "configuration": convert_and_respect_annotation_metadata(
                    object_=configuration, annotation=PatchRateLimitPolicyConfigurationRequestContent, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateRateLimitPolicyResponseContent,
                    parse_obj_as(
                        type_=UpdateRateLimitPolicyResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRateLimitPoliciesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        resource: typing.Optional[RateLimitPolicyResourceEnum] = None,
        consumer: typing.Optional[RateLimitPolicyConsumerEnum] = None,
        consumer_selector: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        from_: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[RateLimitPolicy, ListRateLimitPoliciesPaginatedResponseContent]:
        """
        Parameters
        ----------
        resource : typing.Optional[RateLimitPolicyResourceEnum]
            The API protected by the Rate Limit Policy.

        consumer : typing.Optional[RateLimitPolicyConsumerEnum]
            The consumer to which the rate limit policy applies.

        consumer_selector : typing.Optional[str]
            Identifier or category within the consumer to which the policy applies. Supported values: `client_id:<client_id>` to target a specific client by ID, `client_id:<cimd_uri>` to target a CIMD client by URI, `cimd_clients` to target all CIMD clients, `third_party_clients` to target all third-party clients, or `default` to apply the policy to any consumer identifier not otherwise explicitly targeted.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        from_ : typing.Optional[str]
            Cursor for pagination.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[RateLimitPolicy, ListRateLimitPoliciesPaginatedResponseContent]
            Rate limit policies retrieved successfully.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "rate-limit-policies",
            method="GET",
            params={
                "resource": resource,
                "consumer": consumer,
                "consumer_selector": consumer_selector,
                "take": take,
                "from": from_,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListRateLimitPoliciesPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListRateLimitPoliciesPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.rate_limit_policies
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""

                async def _get_next():
                    return await self.list(
                        resource=resource,
                        consumer=consumer,
                        consumer_selector=consumer_selector,
                        take=take,
                        from_=_parsed_next,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        resource: RateLimitPolicyResourceEnum,
        consumer: RateLimitPolicyConsumerEnum,
        consumer_selector: str,
        configuration: RateLimitPolicyConfiguration,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateRateLimitPolicyResponseContent]:
        """
        Parameters
        ----------
        resource : RateLimitPolicyResourceEnum

        consumer : RateLimitPolicyConsumerEnum

        consumer_selector : str
            Identifier or category within the consumer to which the policy applies. Supported values: `client_id:<client_id>` to target a specific client by ID, `client_id:<cimd_uri>` to target a CIMD client by URI, `cimd_clients` to target all CIMD clients, `third_party_clients` to target all third-party clients, or `default` to apply the policy to any consumer identifier not otherwise explicitly targeted.

        configuration : RateLimitPolicyConfiguration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateRateLimitPolicyResponseContent]
            Rate limit policy successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "rate-limit-policies",
            method="POST",
            json={
                "resource": resource,
                "consumer": consumer,
                "consumer_selector": consumer_selector,
                "configuration": convert_and_respect_annotation_metadata(
                    object_=configuration, annotation=RateLimitPolicyConfiguration, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateRateLimitPolicyResponseContent,
                    parse_obj_as(
                        type_=CreateRateLimitPolicyResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetRateLimitPolicyResponseContent]:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetRateLimitPolicyResponseContent]
            Rate limit policy retrieved successfully.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"rate-limit-policies/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetRateLimitPolicyResponseContent,
                    parse_obj_as(
                        type_=GetRateLimitPolicyResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"rate-limit-policies/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        configuration: PatchRateLimitPolicyConfigurationRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateRateLimitPolicyResponseContent]:
        """
        Parameters
        ----------
        id : str
            Unique identifier for the Rate Limit Policy.

        configuration : PatchRateLimitPolicyConfigurationRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateRateLimitPolicyResponseContent]
            Rate limit policy successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"rate-limit-policies/{encode_path_param(id)}",
            method="PATCH",
            json={
                "configuration": convert_and_respect_annotation_metadata(
                    object_=configuration, annotation=PatchRateLimitPolicyConfigurationRequestContent, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateRateLimitPolicyResponseContent,
                    parse_obj_as(
                        type_=UpdateRateLimitPolicyResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
